<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2007 Tim Lochmueller, Sareen Millet (webmaster@fruit-lab.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Extends tslib_cObj to change the path for the images
 *
 */
class ux_tslib_cObj extends tslib_cObj {

        /**
         * Returns a <img> tag with the image file defined by $file and processed according to the properties in the TypoScript array.
         * Mostly this function is a sub-function to the IMAGE function which renders the IMAGE cObject in TypoScript. This function is called by "$this->cImage($conf['file'],$conf);" from IMAGE().
         *
         * @param        string                File TypoScript resource
         * @param        array                TypoScript configuration properties
         * @return        string                <img> tag, (possibly wrapped in links and other HTML) if any image found.
         * @access private
         * @see IMAGE()
         */
        function cImage($file,$conf) {
            $this->conf['spaceCharacter'] = '-';
                 $info = $this->getImgResource($file,$conf['file.']);
                $GLOBALS['TSFE']->lastImageInfo=$info;
                if (is_array($info))        {
                        $info[3] = t3lib_div::png_to_gif_by_imagemagick($info[3]);
                        $GLOBALS['TSFE']->imagesOnPage[]=$info[3];                // This array is used to collect the image-refs on the page...

                        if (!strlen($conf['altText']) && !is_array($conf['altText.']))        {        // Backwards compatible:
                                $conf['altText'] = $conf['alttext'];
                                $conf['altText.'] = $conf['alttext.'];
                        }

                        // Here begins RealURL Image

                        // if there is no alt-attribut for the <img>-tag use the page title
                        if (!$conf['altText'] || trim($conf['altText']) == "")
                            $conf['altText'] = $this->getLangTitle();

                        // generate the attributes for the <img>-tag
                        $altParam = $this->getAltParam($conf);

                        // check for possible information text about the picture as a base to generate a realUrl path for the picture
                        $text = trim($this->stdWrap($conf['fileName'], $conf['fileName.']));                     // new TypoScript Object!!!!
                        if (!$text)      $text = trim($this->stdWrap($conf['altText'], $conf['altText.']));
                        if (!$text)      $text = trim($this->stdWrap($conf['titleText'],$conf['titleText.']));
                        if (!$text)      $text = trim($this->stdWrap($conf['longdescURL'],$conf['longdescURL.']));
                        if (!$text)      $text = "pic";

                        // the path to the real image location
                        $img_path = htmlspecialchars(trim(t3lib_div::rawUrlEncodeFP($info[3]))); // Das ist der Pfad, der normal zum Bild fhrt: typo3temp/pics/2305e38d9c.jpg

                        // generate a RealUrl-filename for the picture and set the pair picture - realUrl in the DB
                        if ($img_path != "")
                            $img_path = $this->makeImgPath($text,$img_path);

                        // generate the <img>-tag
                        $theValue = '<img src="'.htmlspecialchars($GLOBALS['TSFE']->absRefPrefix).$img_path.'" width="'.$info[0].'" height="'.$info[1].'"'.$this->getBorderAttr(' border="'.intval($conf['border']).'"').($conf['params']?' '.$conf['params']:'').($altParam).' />';

                        // Here ends RealURL Image


                        if ($conf['linkWrap'])        {
                                $theValue = $this->linkWrap($theValue,$conf['linkWrap']);
                        } elseif ($conf['imageLinkWrap']) {
                                $theValue = $this->imageLinkWrap($theValue,$info['origFile'],$conf['imageLinkWrap.']);
                        }
                        return $this->wrap($theValue,$conf['wrap']);
                }
        }

        /*******************************
         *
         * Helper functions
         *
         ******************************/

        /**
         * Generate the RealUrl Path to an image:
         * - Converts a text into a RealUrl path
         * - stores the pair RealUrl and true file location in the DB
         *
         * @param        string                Text used as a basis for the new RealUrl
         *               string                the true file location of the image
         * @return       string                The ready RealUrl Path for the image
         * @see rootLineToPath()
         */
        function makeImgPath($text,$img_path)
        {        // $text      : any text that describes the picture this is the basis for the generation of the filename
            // $img_path  : orginal path to picture z.B. typo3temp/pics/2305e38d9c.jpg

                        $realurl_base = $this->encodeTitle($text);               // file name for possible realUrl
                        $realurl_post = 0;                                       // a postfix to get a unique realUrl
            list($img_base,$realurl_end) = explode('.',basename($img_path));
            $realurl_end = ".".$realurl_end;

            // if there is nothing sensible for a filename use a default (alternative: "return $img_path;" could make sense)
            if ($realurl_base == '')    $realurl_base = $img_base;

            do { // generate a proper realUrl and post it into the DB
            

                            if ($realurl_post == 0)
                                $res_realurl = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid,image_path','tx_flrealurlimagecache','realurl_path=\''.$realurl_base.$realurl_end.'\'');
                            else
                                $res_realurl = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid,image_path','tx_flrealurlimagecache','realurl_path=\''.$realurl_base.'-'.$realurl_post.$realurl_end.'\'');
                            $num_realurl = $GLOBALS['TYPO3_DB']->sql_num_rows($res_realurl);
                            $data_realurl = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res_realurl);

                            if ($num_realurl == 0)
                            {   // URL not jet in DB => set new
                                  if ($realurl_post == 0)      $realurl = $realurl_base.$realurl_end;
                                  else                         $realurl = $realurl_base.'-'.$realurl_post.$realurl_end;
                                  $insertArray = array(
                                      'crdate' => time(),
                                      'image_path' => $img_path,
                                      'realurl_path' => $realurl,
                                  );
                                  $res_realurl = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_flrealurlimagecache', $insertArray);
                                  break;
                            }
                            elseif ($num_realurl == 1 && $data_realurl['image_path'] == $img_path)     break;           // URL with same picture already in DB => nothing to do
                            else                                                                       $realurl_post++;    // URL taken by other picture          => look for free RealURL
                            
                        } while ($num_realurl == 1 && $data_realurl['image_path'] != $img_path);

                        // Return the proper RealUrl
                        if ($realurl_post == 0)                   return $realurl_base.$realurl_end;
                        else                                      return $realurl_base.'-'.$realurl_post.$realurl_end;
        }

        /**
         * Get the titile of the page for the current language:
         *
         * @return       string                the title of the page for the current language
         */
        function getLangTitle() {
              $lang = t3lib_div::GPvar('L');
            if ($lang == 0) {
              $res_page_title = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
                'title',                      // fields
                'pages',                      // table
                'uid='.$GLOBALS['TSFE']->id   // where-condition
              );
              $text_temp = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res_page_title);
              $title = $text_temp['title'];
            }
            else {
              $res_page_title = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
                'title',                      // fields
                'pages_language_overlay',     // table
                  'pid='.$GLOBALS['TSFE']->id.' AND sys_language_uid='.$lang
                 );
              $text_temp = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res_page_title);
              $title = $text_temp['title'];
            }
            return $title;
         }

        /**
         * Convert a title to something that can be used in an page path:
         * - Convert spaces to underscores
         * - Convert non A-Z characters to ASCII equivalents
         * - Convert some special things like the 'ae'-character
         * - Strip off all other symbols
         * Works with the character set defined as "forceCharset"
         *
         * @param        string                Input title to clean
         * @return       string                Encoded title, passed through rawurlencode() = ready to put in the URL.
         * @see rootLineToPath()
         */
        function encodeTitle($title) {

                // Fetch character set:
                $charset = $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] ? $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] : $GLOBALS['TSFE']->defaultCharSet;

                // Convert to lowercase:
                $processedTitle = $GLOBALS['TSFE']->csConvObj->conv_case($charset,$title,'toLower');

                // Convert some special tokens to the space character:
                $space = $this->conf['spaceCharacter'] ? $this->conf['spaceCharacter'] : '-';
                $processedTitle = strtr($processedTitle,' -+_',$space.$space.$space.$space);        // convert spaces

                // Convert extended letters to ascii equivalents:
                $processedTitle = $GLOBALS['TSFE']->csConvObj->specCharsToASCII($charset,$processedTitle);

                // Strip the rest...:
                $processedTitle = ereg_replace('[^a-zA-Z0-9\\'.$space.']', $space, $processedTitle);  // replace other characters with $space
                $processedTitle = ereg_replace('\\'.$space.'+',$space,$processedTitle);               // Convert multiple 'spaces' to a single one
                $processedTitle = trim($processedTitle,$space);

                if ($this->conf['encodeTitle_userProc'])        {
                        $params = array(
                                'pObj' => &$this,
                                'title' => $title,
                                'processedTitle' => $processedTitle,
                        );
                        $processedTitle = t3lib_div::callUserFunction($this->conf['encodeTitle_userProc'], $params, $this);
                }
                
                // Return encoded URL:
                return rawurlencode($processedTitle);
        }

}
?>